#include "monWidgetGL.hpp"

#include <GL/glu.h>
#include <iostream>

using cpe::vec2;
using cpe::vec3;

monWidgetGL::monWidgetGL(QGLWidget *parent) :
    QGLWidget(parent)
{

    QWidget::setFocusPolicy(Qt::StrongFocus);
    etat_affichage=true;
    identifiant_texture=0;

    translation=vec3(0,0,0);
    compteur_temps=0;

    startTimer(25);

}

monWidgetGL::~monWidgetGL()
{}

void monWidgetGL::initializeGL()
{

    std::cout<<"appel monWidgetGL::initializeGL()"<<std::endl;

    glClearColor(1.0, 1.0, 1.0, 0.0);


    vec3 tableau_sommet[4]={vec3(-5,-0.1,-5),
                            vec3(5,-0.1,-5),
                            vec3(5,-0.1, 5),
                            vec3(-5,-0.1, 5)};
    vec2 tableau_uv[4]={vec2(0,0) , vec2(0,1) , vec2(1,1) , vec2(1,0)};
    std::copy(tableau_sommet,tableau_sommet+4*3,std::back_inserter(sommets));
    std::copy(tableau_uv,tableau_uv+4*2,std::back_inserter(uv_texture));

    vec3 tableau_sommet_2[4]={vec3(-0.1,0.2,-1),
                              vec3(-0.1,0.5,-1),
                              vec3( 0.1,0.5,-1),
                              vec3( 0.1,0.2,-1)};
    std::copy(tableau_sommet_2,tableau_sommet_2+4*3,std::back_inserter(sommets_2));
    sommets_2_originaux=sommets_2;

    identifiant_texture=bindTexture(QString("data/texture.jpg"),GL_TEXTURE_2D);
    if(identifiant_texture==0)
        std::cout<<"Chargement texture echouee"<<std::endl;



    glEnable(GL_DEPTH_TEST);
    glDepthMask(GL_TRUE);

}

void monWidgetGL::paintGL()
{

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);


    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60,1.0,0.1,10.0);
    glRotated(180.0*angle/M_PI,0,1,0);
    glTranslated(translation.x(),translation.y(),translation.z());


    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();




    if(etat_affichage==true)
    {
        glColor3d(1,1,1);
        glNormal3d(0,-1,0);

        glEnable(GL_TEXTURE);
        glEnable(GL_TEXTURE_2D);
        glBindTexture(GL_TEXTURE_2D,identifiant_texture);





        glEnableClientState(GL_VERTEX_ARRAY);
        glEnableClientState(GL_TEXTURE_COORD_ARRAY);
        glVertexPointer(3,GL_FLOAT,0,sommets[0].pointer());
        glTexCoordPointer(2,GL_FLOAT,0,uv_texture[0].pointer());
        glDrawArrays(GL_QUADS,0,sommets.size()/3);
        glDisableClientState(GL_VERTEX_ARRAY);
        glDisableClientState(GL_TEXTURE_COORD_ARRAY);
        glDisable(GL_TEXTURE_2D);


        double temps=static_cast<double>(compteur_temps)/1000.0;
        vec3 deplacement=vec3(0.3*sin(0.1*sin(5*2*M_PI*temps)),0.5*cos(10*2*M_PI*temps),0);
        for(unsigned int k=0;k<sommets_2.size();++k)
            sommets_2[k]=sommets_2_originaux[k]+deplacement;

        glNormal3d(0,0,1);
        glColor3d(1,0,0);
        glEnableClientState(GL_VERTEX_ARRAY);
        glVertexPointer(3,GL_FLOAT,0,sommets_2[0].pointer());
        glDrawArrays(GL_QUADS,0,sommets_2.size()/3);
        glDisableClientState(GL_VERTEX_ARRAY);
    }


}



void monWidgetGL::afficher()
{
    etat_affichage=!etat_affichage;

    updateGL();
}
void monWidgetGL::wireframe(bool est_actif)
{
    if(est_actif==true)
        glPolygonMode( GL_FRONT_AND_BACK, GL_LINE);
    else
        glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);

    updateGL();
}

void monWidgetGL::keyPressEvent(QKeyEvent *event)
{

    double d_angle=0.1;
    double d_radial=0.1;
    double d_height=0.1;

    vec3 d_tr(-d_radial*sin(angle),0,d_radial*cos(angle));
    int current=event->key();


    switch(current)
    {
    case Qt::Key_Up:
        translation += d_tr;
        break;
    case Qt::Key_Down:
        translation -= d_tr;
        break;
    case Qt::Key_Left:
        angle-=d_angle;
        break;
    case Qt::Key_Right:
        angle+=d_angle;
        break;

    case Qt::Key_A:
        translation.y() -= d_height;
        break;
    case Qt::Key_Q:
        translation.y() += d_height;
        break;
    }

    updateGL();
}

void monWidgetGL::timerEvent(QTimerEvent *event)
{

    compteur_temps=(compteur_temps+1)%1000;


    event->accept();
    updateGL();


}

